@@ -1,9 +1,10 @@
 """PGSync Base class."""
-import collections
 import logging
 import os
 import sys
 import warnings
+from collections import defaultdict
+from typing import List, Optional, Tuple
 
 import sqlalchemy as sa
 import sqlparse
@@ -24,6 +25,7 @@ from .constants import (
     UPDATE,
 )
 from .exc import ForeignKeyError, LogicalSlotParseError, TableNotFoundError
+from .node import Node
 from .settings import PG_SSLMODE, PG_SSLROOTCERT, QUERY_CHUNK_SIZE
 from .trigger import CREATE_TRIGGER_TEMPLATE
 from .utils import get_postgres_url
@@ -44,7 +46,7 @@ logger = logging.getLogger(__name__)
 
 
 class Base(object):
-    def __init__(self, database, verbose=False, *args, **kwargs):
+    def __init__(self, database: str, verbose: bool = False, *args, **kwargs):
         """Initialize the base class constructor.
 
         Args:
@@ -53,9 +55,9 @@ class Base(object):
         self.__engine = pg_engine(database, **kwargs)
         self.__schemas = None
         # models is a dict of f'{schema}.{table}'
-        self.models = {}
-        self.__metadata = {}
-        self.verbose = verbose
+        self.models: dict = {}
+        self.__metadata: dict = {}
+        self.verbose: bool = verbose
 
     def connect(self) -> None:
         """Connect to database."""
@@ -66,7 +68,7 @@ class Base(object):
             logger.exception(f"Cannot connect to database: {e}")
             raise
 
-    def pg_settings(self, column) -> None:
+    def pg_settings(self, column: str) -> Optional[str]:
         try:
             return self.fetchone(
                 sa.select([sa.column("setting")])
@@ -96,8 +98,10 @@ class Base(object):
                 ),
                 label="has_permission",
             )[0]
-        except Exception as e:
+        except (Exception, TypeError) as e:
             logger.exception(f"{e}")
+            return False
+
         return False
 
     # Tables...
@@ -112,7 +116,7 @@ class Base(object):
             The SQLAlchemy aliased model representation
 
         """
-        name = f"{schema}.{table}"
+        name: str = f"{schema}.{table}"
         if name not in self.models:
             if schema not in self.__metadata:
                 metadata = sa.MetaData(schema=schema)
@@ -161,7 +165,7 @@ class Base(object):
                     self.__schemas.remove(schema)
         return self.__schemas
 
-    def tables(self, schema):
+    def tables(self, schema: str) -> List:
         """:obj:`list` of :obj:`str`: Get all tables.
 
         returns the fully qualified table name with schema {schema}.{table}
@@ -173,7 +177,7 @@ class Base(object):
         metadata = self.__metadata[schema]
         return metadata.tables.keys()
 
-    def _get_schema(self, schema, table):
+    def _get_schema(self, schema: str, table: str) -> Tuple[str, str]:
         pairs = table.split(".")
         if len(pairs) == 2:
             return pairs[0], pairs[1]
@@ -181,7 +185,7 @@ class Base(object):
             return schema, pairs[0]
         raise ValueError(f"Invalid definition {table} for schema: {schema}")
 
-    def truncate_table(self, table, schema=SCHEMA):
+    def truncate_table(self, table: str, schema: str = SCHEMA) -> None:
         """Truncate a table.
 
         Note:
@@ -200,18 +204,18 @@ class Base(object):
         query = f'TRUNCATE TABLE "{schema}"."{table}" CASCADE'
         self.execute(query)
 
-    def truncate_tables(self, tables, schema=SCHEMA):
+    def truncate_tables(self, tables: List[str], schema: str = SCHEMA) -> None:
         """Truncate all tables."""
         logger.debug(f"Truncating tables: {tables}")
         for table in tables:
             self.truncate_table(table, schema=schema)
 
-    def truncate_schema(self, schema):
+    def truncate_schema(self, schema: str) -> None:
         """Truncate all tables in a schema."""
         logger.debug(f"Truncating schema: {schema}")
         self.truncate_tables(self.tables(schema), schema=schema)
 
-    def truncate_schemas(self):
+    def truncate_schemas(self) -> None:
         """Truncate all tables in a database."""
         for schema in self.schemas:
             self.truncate_schema(schema)
@@ -219,10 +223,10 @@ class Base(object):
     # Replication slots...
     def replication_slots(
         self,
-        slot_name,
-        plugin=PLUGIN,
-        slot_type="logical",
-    ):
+        slot_name: str,
+        plugin: str = PLUGIN,
+        slot_type: str = "logical",
+    ) -> List[str]:
         """List replication slots.
 
         SELECT * FROM PG_REPLICATION_SLOTS
@@ -242,7 +246,7 @@ class Base(object):
             label="replication_slots",
         )
 
-    def create_replication_slot(self, slot_name):
+    def create_replication_slot(self, slot_name: str) -> None:
         """Create a replication slot.
 
         TODO:
@@ -262,7 +266,7 @@ class Base(object):
             label="create_replication_slot",
         )
 
-    def drop_replication_slot(self, slot_name):
+    def drop_replication_slot(self, slot_name: str) -> None:
         """Drop a replication slot."""
         logger.debug(f"Dropping replication slot: {slot_name}")
         if self.replication_slots(slot_name):
@@ -279,15 +283,15 @@ class Base(object):
 
     def _logical_slot_changes(
         self,
-        slot_name,
-        func,
-        txmin=None,
-        txmax=None,
-        upto_lsn=None,
-        upto_nchanges=None,
+        slot_name: str,
+        func: sa.sql.functions._FunctionGenerator,
+        txmin: Optional[int] = None,
+        txmax: Optional[int] = None,
+        upto_lsn: Optional[int] = None,
+        upto_nchanges: Optional[int] = None,
     ):
-        filters = []
-        statement = sa.select(
+        filters: List = []
+        statement: str = sa.select(
             [sa.column("xid"), sa.column("data")]
         ).select_from(
             func(
@@ -318,11 +322,11 @@ class Base(object):
 
     def logical_slot_get_changes(
         self,
-        slot_name,
-        txmin=None,
-        txmax=None,
-        upto_lsn=None,
-        upto_nchanges=None,
+        slot_name: str,
+        txmin: Optional[int] = None,
+        txmax: Optional[int] = None,
+        upto_lsn: Optional[int] = None,
+        upto_nchanges: Optional[int] = None,
     ):
         """Get/Consume changes from a logical replication slot.
 
@@ -343,11 +347,11 @@ class Base(object):
 
     def logical_slot_peek_changes(
         self,
-        slot_name,
-        txmin=None,
-        txmax=None,
-        upto_lsn=None,
-        upto_nchanges=None,
+        slot_name: str,
+        txmin: Optional[int] = None,
+        txmax: Optional[int] = None,
+        upto_lsn: Optional[int] = None,
+        upto_nchanges: Optional[int] = None,
     ):
         """Peek a logical replication slot without consuming changes.
 
@@ -363,7 +367,7 @@ class Base(object):
         )
 
     # Views...
-    def _primary_keys(self, schema, tables):
+    def _primary_keys(self, schema: str, tables: List[str]):
         with warnings.catch_warnings():
             warnings.simplefilter("ignore", category=sa.exc.SAWarning)
             pg_class = self.model("pg_class", "pg_catalog")
@@ -437,7 +441,7 @@ class Base(object):
             .group_by(pg_index.c.indrelid)
         )
 
-    def _foreign_keys(self, schema, tables):
+    def _foreign_keys(self, schema: str, tables: List[str]):
         with warnings.catch_warnings():
             warnings.simplefilter("ignore", category=sa.exc.SAWarning)
             table_constraints = self.model(
@@ -605,13 +609,15 @@ class Base(object):
         )
         logger.debug(f"Created view: {schema}.{MATERIALIZED_VIEW}")
 
-    def drop_view(self, schema):
+    def drop_view(self, schema: str):
         logger.debug(f"Dropping view: {schema}.{MATERIALIZED_VIEW}")
         self.__engine.execute(DropView(schema, MATERIALIZED_VIEW))
         logger.debug(f"Dropped view: {schema}.{MATERIALIZED_VIEW}")
 
     # Triggers...
-    def create_triggers(self, schema, tables=None):
+    def create_triggers(
+        self, schema: str, tables: Optional[List[str]] = None
+    ) -> None:
         """Create a database triggers."""
         self.execute(
             CREATE_TRIGGER_TEMPLATE.replace(
@@ -642,7 +648,9 @@ class Base(object):
         for query in queries:
             self.execute(query)
 
-    def drop_triggers(self, schema, tables=None):
+    def drop_triggers(
+        self, schema: str, tables: Optional[List[str]] = None
+    ) -> None:
         """Drop all pgsync defined triggers in database."""
         for table in self.tables(schema):
             schema, table = self._get_schema(schema, table)
@@ -656,7 +664,7 @@ class Base(object):
                 )
                 self.execute(query)
 
-    def disable_triggers(self, schema):
+    def disable_triggers(self, schema: str) -> None:
         """Disable all pgsync defined triggers in database."""
         for table in self.tables(schema):
             schema, table = self._get_schema(schema, table)
@@ -668,7 +676,7 @@ class Base(object):
                 )
                 self.execute(query)
 
-    def enable_triggers(self, schema):
+    def enable_triggers(self, schema: str) -> None:
         """Enable all pgsync defined triggers in database."""
         for table in self.tables(schema):
             schema, table = self._get_schema(schema, table)
@@ -692,7 +700,7 @@ class Base(object):
             label="txid_current",
         )[0]
 
-    def parse_value(self, type_, value):
+    def parse_value(self, type_: str, value: str) -> Optional[str]:
         """
         Parse datatypes from db.
 
@@ -717,7 +725,7 @@ class Base(object):
             "smallserial",
         ):
             try:
-                value = int(value)
+                value: int = int(value)
             except ValueError:
                 raise
         if type_.lower() in (
@@ -728,10 +736,10 @@ class Base(object):
             "uuid",
             "varchar",
         ):
-            value = value.lstrip("'").rstrip("'")
+            value: str = value.lstrip("'").rstrip("'")
         if type_.lower() == "boolean":
             try:
-                value = bool(value)
+                value: bool = bool(value)
             except ValueError:
                 raise
         if type_.lower() in (
@@ -741,7 +749,7 @@ class Base(object):
             "real",
         ):
             try:
-                value = float(value)
+                value: float = float(value)
             except ValueError:
                 raise
         return value
@@ -893,7 +901,7 @@ def subtransactions(session):
 
 def _get_foreign_keys(model_a, model_b):
 
-    foreign_keys = collections.defaultdict(list)
+    foreign_keys = defaultdict(list)
 
     if model_a.foreign_keys:
 
@@ -921,7 +929,7 @@ def _get_foreign_keys(model_a, model_b):
     return foreign_keys
 
 
-def get_foreign_keys(node_a, node_b):
+def get_foreign_keys(node_a: Node, node_b: Node) -> dict:
     """Return dict of single foreign key with multiple columns.
 
     e.g:
@@ -932,7 +940,7 @@ def get_foreign_keys(node_a, node_b):
 
     column_1, column_2, column_N are of type ForeignKeyContraint
     """
-    foreign_keys = {}
+    foreign_keys: dict = {}
     # if either offers a foreign_key via relationship, use it!
     if (
         node_a.relationship.foreign_key.parent
@@ -972,8 +980,8 @@ def pg_engine(
     sslrootcert=None,
 ):
     connect_args = {}
-    sslmode = sslmode or PG_SSLMODE
-    sslrootcert = sslrootcert or PG_SSLROOTCERT
+    sslmode: str = sslmode or PG_SSLMODE
+    sslrootcert: str = sslrootcert or PG_SSLROOTCERT
 
     if sslmode:
         if sslmode not in (
@@ -996,7 +1004,7 @@ def pg_engine(
             )
         connect_args["sslrootcert"] = sslrootcert
 
-    url = get_postgres_url(
+    url: str = get_postgres_url(
         database,
         user=user,
         host=host,
@@ -1006,8 +1014,8 @@ def pg_engine(
     return sa.create_engine(url, echo=echo, connect_args=connect_args)
 
 
-def pg_execute(engine, query, values=None, options=None):
-    options = options or {"isolation_level": "AUTOCOMMIT"}
+def pg_execute(engine, query, values=None, options=None) -> None:
+    options: dict = options or {"isolation_level": "AUTOCOMMIT"}
     conn = engine.connect()
     try:
         if options:
@@ -1019,13 +1027,13 @@ def pg_execute(engine, query, values=None, options=None):
         raise
 
 
-def create_schema(engine, schema):
+def create_schema(engine, schema) -> None:
     """Create database schema."""
     if schema != SCHEMA:
         engine.execute(sa.schema.CreateSchema(schema))
 
 
-def create_database(database, echo=False):
+def create_database(database: str, echo: bool = False) -> None:
     """Create a database."""
     logger.debug(f"Creating database: {database}")
     engine = pg_engine(database="postgres", echo=echo)
@@ -1033,7 +1041,7 @@ def create_database(database, echo=False):
     logger.debug(f"Created database: {database}")
 
 
-def drop_database(database, echo=False):
+def drop_database(database: str, echo: bool = False) -> None:
     """Drop a database."""
     logger.debug(f"Dropping database: {database}")
     engine = pg_engine(database="postgres", echo=echo)
@@ -1041,7 +1049,9 @@ def drop_database(database, echo=False):
     logger.debug(f"Dropped database: {database}")
 
 
-def create_extension(database, extension, echo=False):
+def create_extension(
+    database: str, extension: str, echo: bool = False
+) -> None:
     """Create a database extension."""
     logger.debug(f"Creating extension: {extension}")
     engine = pg_engine(database=database, echo=echo)
@@ -1049,7 +1059,7 @@ def create_extension(database, extension, echo=False):
     logger.debug(f"Created extension: {extension}")
 
 
-def drop_extension(database, extension, echo=False):
+def drop_extension(database: str, extension: str, echo: bool = False) -> None:
     """Drop a database extension."""
     logger.debug(f"Dropping extension: {extension}")
     engine = pg_engine(database=database, echo=echo)
@@ -1057,15 +1067,17 @@ def drop_extension(database, extension, echo=False):
     logger.debug(f"Dropped extension: {extension}")
 
 
-def compiled_query(query, label=None, literal_binds=False):
+def compiled_query(
+    query: str, label: Optional[str] = None, literal_binds: bool = False
+) -> None:
     """Compile an SQLAlchemy query with an optional label."""
-    query = str(
+    query: str = str(
         query.compile(
             dialect=postgresql.dialect(),
             compile_kwargs={"literal_binds": literal_binds},
         )
     )
-    query = sqlparse.format(query, reindent=True, keyword_case="upper")
+    query: str = sqlparse.format(query, reindent=True, keyword_case="upper")
     if label:
         logger.debug(f"\033[4m{label}:\033[0m\n{query}")
         sys.stdout.write(f"\033[4m{label}:\033[0m\n{query}")
