@@ -1,10 +1,11 @@
 """PGSync plugin."""
-import importlib
-import inspect
 import logging
 import os
-import pkgutil
 from abc import ABC, abstractmethod
+from importlib import import_module
+from inspect import getmembers, isclass
+from pkgutil import iter_modules
+from typing import Optional
 
 logger = logging.getLogger(__name__)
 
@@ -13,36 +14,36 @@ class Plugin(ABC):
     """Plugin base class."""
 
     @abstractmethod
-    def transform(self, doc, **kwargs):
+    def transform(self, doc: list, **kwargs) -> dict:
         """Must be implemented by all derived classes."""
         pass
 
 
 class Plugins(object):
-    def __init__(self, package, names=None):
-        self.package = package
-        self.names = names or []
+    def __init__(self, package: str, names: Optional[list] = None):
+        self.package: str = package
+        self.names: Optional[list] = names or []
         self.reload()
 
-    def reload(self):
+    def reload(self) -> None:
         """Reload the plugins from the available list."""
-        self.plugins = []
-        self._paths = []
+        self.plugins: list = []
+        self._paths: list = []
         logger.debug(f"Reloading plugins from package: {self.package}")
         self.walk(self.package)
 
-    def walk(self, package):
+    def walk(self, package: str) -> None:
         """Recursively walk the supplied package and fetch all plugins"""
-        plugins = importlib.import_module(package)
-        for _, name, ispkg in pkgutil.iter_modules(
+        plugins = import_module(package)
+        for _, name, ispkg in iter_modules(
             plugins.__path__,
             f"{plugins.__name__}.",
         ):
             if ispkg:
                 continue
 
-            module = importlib.import_module(name)
-            members = inspect.getmembers(module, inspect.isclass)
+            module = import_module(name)
+            members = getmembers(module, isclass)
             for _, klass in members:
                 if issubclass(klass, Plugin) & (klass is not Plugin):
                     if klass.name not in self.names:
@@ -52,7 +53,7 @@ class Plugins(object):
                     )
                     self.plugins.append(klass())
 
-        paths = []
+        paths: list = []
         if isinstance(plugins.__path__, str):
             paths.append(plugins.__path__)
         else:
@@ -71,7 +72,7 @@ class Plugins(object):
             ]:
                 self.walk(f"{package}.{pkg}")
 
-    def transform(self, docs):
+    def transform(self, docs: list) -> dict:
         """Apply all plugins to each doc."""
         for doc in docs:
             for plugin in self.plugins:
@@ -82,3 +83,14 @@ class Plugins(object):
                     _index=doc["_index"],
                 )
             yield doc
+
+    def auth(self, key: str) -> Optional[str]:
+        """Get an auth value from a key."""
+        for plugin in self.plugins:
+            if hasattr(plugin, "auth"):
+                logger.debug(f"Plugin: {plugin.name}")
+                try:
+                    return plugin.auth(key)
+                except Exception as e:
+                    logger.exception(f"Error calling auth: {e}")
+                    return None
