@@ -1,5 +1,5 @@
 """PGSync QueryBuilder."""
-from typing import Dict
+from typing import Dict, List
 
 import sqlalchemy as sa
 
@@ -15,15 +15,10 @@ class QueryBuilder(object):
     def __init__(self, base: Base, verbose: bool = False):
         """
         Query builder constructor.
-
-        :param base: base class
-        :type base: `base.class`
-        :param verbose: verbose
-        :type verbose: `bool`
         """
-        self.base = base
-        self.verbose = verbose
-        self.isouter = True
+        self.base: Base = base
+        self.verbose: bool = verbose
+        self.isouter: bool = True
 
     def _get_foreign_keys(self, node_a: Node, node_b: Node) -> Dict:
         if (
@@ -38,12 +33,12 @@ class QueryBuilder(object):
                 through_tables = node_b.relationship.through_tables[0]
                 node_a, node_b = node_b, node_a
 
-            through = node_from_table(
+            through: Node = node_from_table(
                 self.base,
                 through_tables,
                 node_a.schema,
             )
-            foreign_keys = get_foreign_keys(node_a, through)
+            foreign_keys: dict = get_foreign_keys(node_a, through)
             for key, value in get_foreign_keys(through, node_b).items():
                 if key in foreign_keys:
                     foreign_keys[key].extend(value)
@@ -77,9 +72,9 @@ class QueryBuilder(object):
         """
         # TODO: normalize this elsewhere
         try:
-            column_names = [column.name for column in columns]
+            column_names: List = [column.name for column in columns]
         except AttributeError:
-            column_names = [column for column in columns]
+            column_names: List = [column for column in columns]
 
         if table is None:
             for table, cols in foreign_keys.items():
@@ -94,14 +89,14 @@ class QueryBuilder(object):
                     foreign_keys[table].pop(i)
             return foreign_keys[table]
 
-        msg = (
+        msg: str = (
             f"No keys for columns: {columns} and foreign_keys: {foreign_keys}"
         )
         if table:
             msg += f" with table {table}"
         raise FetchColumnForeignKeysError(msg)
 
-    def _get_child_keys(self, node, params):
+    def _get_child_keys(self, node: Node, params: dict):
         row = sa.cast(
             sa.func.JSON_BUILD_OBJECT(
                 node.table,
@@ -153,11 +148,11 @@ class QueryBuilder(object):
             node._subquery = node._subquery.where(sa.and_(*node._filters))
         node._subquery = node._subquery.alias()
 
-    def _children(self, node):
+    def _children(self, node: Node):
 
         for child in node.children:
 
-            onclause = []
+            onclause: List = []
 
             if child.relationship.through_tables:
 
@@ -285,7 +280,7 @@ class QueryBuilder(object):
                 isouter=self.isouter,
             )
 
-    def _through(self, node):  # noqa: C901
+    def _through(self, node: Node):  # noqa: C901
 
         through = node_from_table(
             self.base,
@@ -572,7 +567,7 @@ class QueryBuilder(object):
 
         node._subquery = subquery.alias()
 
-    def _non_through(self, node):  # noqa: C901
+    def _non_through(self, node: Node):  # noqa: C901
 
         from_obj = None
 
@@ -692,7 +687,7 @@ class QueryBuilder(object):
                 node, sa.func.JSON_AGG(sa.func.JSON_BUILD_OBJECT(*params))
             )
 
-        columns = [_keys]
+        columns: List = [_keys]
 
         if node.relationship.variant == SCALAR:
             # TODO: Raise exception here if the number of columns > 1
@@ -753,7 +748,7 @@ class QueryBuilder(object):
 
         node._subquery = node._subquery.alias()
 
-    def build_queries(self, node):
+    def build_queries(self, node: Node):
         """Build node query."""
         self.from_obj = None
 
