@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 
 """Main module."""
-import collections
 import json
 import logging
 import os
@@ -10,8 +9,10 @@ import re
 import select
 import sys
 import time
+import uuid
+from collections import defaultdict
 from datetime import datetime, timedelta
-from typing import Generator, List, Optional
+from typing import AnyStr, Generator, List, Optional, Set
 
 import click
 import psycopg2
@@ -42,6 +43,7 @@ from .plugin import Plugins
 from .querybuilder import QueryBuilder
 from .redisqueue import RedisQueue
 from .settings import (
+    CHECKPOINT_PATH,
     POLL_TIMEOUT,
     REDIS_POLL_INTERVAL,
     REPLICATION_SLOT_CLEANUP_INTERVAL,
@@ -59,12 +61,11 @@ class Sync(Base):
         self,
         document: dict,
         verbose: Optional[bool] = False,
-        params: Optional[dict] = None,
         validate: Optional[bool] = True,
         repl_slots: Optional[bool] = True,
+        **kwargs,
     ):
         """Constructor."""
-        params: dict = params or {}
         self.index: str = document["index"]
         self.pipeline: str = document.get("pipeline")
         self.plugins: List = document.get("plugins", [])
@@ -72,16 +73,18 @@ class Sync(Base):
         self.setting: dict = document.get("setting")
         self.routing: str = document.get("routing")
         super().__init__(
-            document.get("database", self.index), verbose=verbose, **params
+            document.get("database", self.index), verbose=verbose, **kwargs
         )
         self.es: ElasticHelper = ElasticHelper()
         self.__name: str = re.sub(
-            "[^0-9a-zA-Z_]+", "", f"{self.database}_{self.index}"
+            "[^0-9a-zA-Z_]+", "", f"{self.database.lower()}_{self.index}"
         )
         self._checkpoint: int = None
         self._plugins: Plugins = None
         self._truncate: bool = False
-        self._checkpoint_file: str = f".{self.__name}"
+        self._checkpoint_file: str = os.path.join(
+            CHECKPOINT_PATH, f".{self.__name}"
+        )
         self.redis: RedisQueue = RedisQueue(self.__name)
         self.tree: Tree = Tree(self)
         self._last_truncate_timestamp: datetime = datetime.now()
@@ -91,6 +94,7 @@ class Sync(Base):
         self.query_builder: QueryBuilder = QueryBuilder(
             self, verbose=self.verbose
         )
+        self.count: dict = dict(db=0, redis=0, elastic=0)
 
     def validate(self, repl_slots: Optional[bool] = True) -> None:
         """Perform all validation right away."""
@@ -103,9 +107,11 @@ class Sync(Base):
 
         self.connect()
         if self.plugins:
-            self._plugins = Plugins("plugins", self.plugins)
+            self._plugins: Plugins = Plugins("plugins", self.plugins)
 
-        max_replication_slots = self.pg_settings("max_replication_slots")
+        max_replication_slots: Optional[str] = self.pg_settings(
+            "max_replication_slots"
+        )
         try:
             if int(max_replication_slots) < 1:
                 raise TypeError
@@ -115,13 +121,15 @@ class Sync(Base):
                 "by setting max_replication_slots=1"
             )
 
-        wal_level = self.pg_settings("wal_level")
+        wal_level: Optional[str] = self.pg_settings("wal_level")
         if not wal_level or wal_level.lower() != "logical":
             raise RuntimeError(
                 "Enable logical decoding by setting wal_level=logical"
             )
 
-        rds_logical_replication = self.pg_settings("rds.logical_replication")
+        rds_logical_replication: Optional[str] = self.pg_settings(
+            "rds.logical_replication"
+        )
 
         if rds_logical_replication:
             if rds_logical_replication.lower() == "off":
@@ -155,14 +163,21 @@ class Sync(Base):
                     f'Make sure you have run the "bootstrap" command.'
                 )
 
-        root = self.tree.build(self.nodes)
+        # ensure the checkpoint dirpath is valid
+        if not os.path.exists(CHECKPOINT_PATH):
+            raise RuntimeError(
+                f'Ensure the checkpoint directory exists "{CHECKPOINT_PATH}" '
+                f"and is readable ."
+            )
+
+        root: Node = self.tree.build(self.nodes)
         root.display()
         for node in traverse_breadth_first(root):
             pass
 
     def create_setting(self) -> None:
         """Create Elasticsearch setting and mapping if required."""
-        root = self.tree.build(self.nodes)
+        root: Node = self.tree.build(self.nodes)
         self.es._create_setting(
             self.index,
             root,
@@ -175,11 +190,11 @@ class Sync(Base):
         self.teardown(drop_view=False)
 
         for schema in self.schemas:
-            tables = set([])
+            tables: Set = set([])
             # tables with user defined foreign keys
-            user_defined_fkey_tables = {}
+            user_defined_fkey_tables: dict = {}
 
-            root = self.tree.build(self.nodes)
+            root: Node = self.tree.build(self.nodes)
             for node in traverse_breadth_first(root):
                 if node.schema != schema:
                     continue
@@ -189,7 +204,7 @@ class Sync(Base):
                 # even though only one of them is the foreign_key.
                 # this is because we define both in the schema but
                 # do not specify which table is the foreign key.
-                columns = []
+                columns: list = []
                 if node.relationship.foreign_key.parent:
                     columns.extend(node.relationship.foreign_key.parent)
                 if node.relationship.foreign_key.child:
@@ -211,8 +226,8 @@ class Sync(Base):
             pass
 
         for schema in self.schemas:
-            tables = set([])
-            root = self.tree.build(self.nodes)
+            tables: Set = set([])
+            root: Node = self.tree.build(self.nodes)
             for node in traverse_breadth_first(root):
                 tables |= set(node.relationship.through_tables)
                 tables |= set([node.table])
@@ -262,9 +277,9 @@ class Sync(Base):
             upto_nchanges=None,
         )
 
-        rows = rows or []
-        payloads = []
-        _rows = []
+        rows: List = rows or []
+        payloads: List = []
+        _rows: List = []
 
         for row in rows:
             if re.search(r"^BEGIN", row.data) or re.search(
@@ -287,7 +302,7 @@ class Sync(Base):
                 raise
             payloads.append(payload)
 
-            j = i + 1
+            j: int = i + 1
             if j < len(_rows):
                 try:
                     payload2 = self.parse_logical_slot(_rows[j].data)
@@ -302,10 +317,10 @@ class Sync(Base):
                     or payload["table"] != payload2["table"]
                 ):
                     self.sync(self._payloads(payloads))
-                    payloads = []
+                    payloads: List = []
             elif j == len(_rows):
                 self.sync(self._payloads(payloads))
-                payloads = []
+                payloads: List = []
 
         if rows:
             self.logical_slot_get_changes(
@@ -332,7 +347,7 @@ class Sync(Base):
             if node.table == root.table:
 
                 for payload in payloads:
-                    payload_data = self._payload_data(payload)
+                    payload_data: dict = self._payload_data(payload)
                     primary_values = [
                         payload_data[key] for key in node.model.primary_keys
                     ]
@@ -359,7 +374,7 @@ class Sync(Base):
                 )
 
                 for payload in payloads:
-                    payload_data = self._payload_data(payload)
+                    payload_data: dict = self._payload_data(payload)
                     for i, key in enumerate(foreign_keys[node.name]):
                         value = payload_data[key]
                         filters[node.parent.table].append(
@@ -377,7 +392,7 @@ class Sync(Base):
             )
 
             for payload in payloads:
-                payload_data = self._payload_data(payload)
+                payload_data: dict = self._payload_data(payload)
                 for i, key in enumerate(foreign_keys[node.name]):
                     value = payload_data[key]
                     filters[node.parent.table].append(
@@ -403,20 +418,20 @@ class Sync(Base):
             #    primary key has changed
             #   2.1) This is crucial otherwise we can have the old
             #        and new document in Elasticsearch at the same time
-            docs = []
+            docs: List = []
             for payload in payloads:
-                payload_data = self._payload_data(payload)
-                primary_values = [
+                payload_data: dict = self._payload_data(payload)
+                primary_values: list = [
                     payload_data[key] for key in node.model.primary_keys
                 ]
-                primary_fields = dict(
+                primary_fields: dict = dict(
                     zip(node.model.primary_keys, primary_values)
                 )
                 filters[node.table].append(
                     {key: value for key, value in primary_fields.items()}
                 )
 
-                old_values = []
+                old_values: list = []
                 for key in root.model.primary_keys:
                     if key in payload.get("old").keys():
                         old_values.append(payload.get("old")[key])
@@ -429,7 +444,7 @@ class Sync(Base):
                     len(old_values) == len(new_values)
                     and old_values != new_values
                 ):
-                    doc = {
+                    doc: dict = {
                         "_id": self.get_doc_id(old_values),
                         "_index": self.index,
                         "_op_type": "delete",
@@ -447,15 +462,15 @@ class Sync(Base):
 
             # update the child tables
             for payload in payloads:
-                _filters = []
-                fields = collections.defaultdict(list)
+                _filters: list = []
+                fields: dict = defaultdict(list)
 
-                payload_data = self._payload_data(payload)
+                payload_data: dict = self._payload_data(payload)
 
-                primary_values = [
+                primary_values: list = [
                     payload_data[key] for key in node.model.primary_keys
                 ]
-                primary_fields = dict(
+                primary_fields: dict = dict(
                     zip(node.model.primary_keys, primary_values)
                 )
 
@@ -478,7 +493,7 @@ class Sync(Base):
 
                 # also handle foreign_keys
                 if node.parent:
-                    fields = collections.defaultdict(list)
+                    fields = defaultdict(list)
                     foreign_keys = get_foreign_keys(
                         node.parent,
                         node,
@@ -499,7 +514,7 @@ class Sync(Base):
                         node.parent.table,
                         fields,
                     ):
-                        where = {}
+                        where: dict = {}
                         params = doc_id.split(PRIMARY_KEY_DELIMITER)
                         for i, key in enumerate(root.model.primary_keys):
                             where[key] = params[i]
@@ -517,13 +532,13 @@ class Sync(Base):
         # when deleting a root node, just delete the doc in Elasticsearch
         if node.table == root.table:
 
-            docs = []
+            docs: list = []
             for payload in payloads:
-                payload_data = self._payload_data(payload)
-                root_primary_values = [
+                payload_data: dict = self._payload_data(payload)
+                root_primary_values: list = [
                     payload_data[key] for key in root.model.primary_keys
                 ]
-                doc = {
+                doc: dict = {
                     "_id": self.get_doc_id(root_primary_values),
                     "_index": self.index,
                     "_op_type": "delete",
@@ -542,14 +557,14 @@ class Sync(Base):
             # the child keys match in private, then get the root doc_id and
             # re-sync the child tables
             for payload in payloads:
-                payload_data = self._payload_data(payload)
-                primary_values = [
+                payload_data: dict = self._payload_data(payload)
+                primary_values: list = [
                     payload_data[key] for key in node.model.primary_keys
                 ]
                 primary_fields = dict(
                     zip(node.model.primary_keys, primary_values)
                 )
-                fields = collections.defaultdict(list)
+                fields = defaultdict(list)
 
                 _filters = []
                 for key, value in primary_fields.items():
@@ -568,13 +583,13 @@ class Sync(Base):
 
         return filters
 
-    def _truncate(self, node: Node, root: Node, filters: dict) -> None:
+    def _truncate(self, node: Node, root: Node, filters: dict) -> dict:
 
         if node.table == root.table:
 
-            docs = []
+            docs: list = []
             for doc_id in self.es._search(self.index, node.table):
-                doc = {
+                doc: dict = {
                     "_id": doc_id,
                     "_index": self.index,
                     "_op_type": "delete",
@@ -587,9 +602,9 @@ class Sync(Base):
 
         else:
 
-            _filters = []
+            _filters: list = []
             for doc_id in self.es._search(self.index, node.table):
-                where = {}
+                where: dict = {}
                 params = doc_id.split(PRIMARY_KEY_DELIMITER)
                 for i, key in enumerate(root.model.primary_keys):
                     where[key] = params[i]
@@ -626,9 +641,9 @@ class Sync(Base):
         ]
 
         """
-        payload = payloads[0]
-        tg_op = payload["tg_op"]
-        table = payload["table"]
+        payload: dict = payloads[0]
+        tg_op: str = payload["tg_op"]
+        table: str = payload["table"]
         if tg_op not in TG_OP:
             logger.exception(f"Unknown tg_op {tg_op}")
             raise
@@ -644,11 +659,11 @@ class Sync(Base):
         ):
             return
 
-        node = get_node(self.tree, table, self.nodes)
-        root = get_node(self.tree, self.nodes["table"], self.nodes)
+        node: Node = get_node(self.tree, table, self.nodes)
+        root: Node = get_node(self.tree, self.nodes["table"], self.nodes)
 
         for payload in payloads:
-            payload_data = self._payload_data(payload)
+            payload_data: dict = self._payload_data(payload)
             # this is only required for the non truncate tg_ops
             if payload_data:
                 if not set(node.model.primary_keys).issubset(
@@ -663,8 +678,8 @@ class Sync(Base):
 
         logger.debug(f"tg_op: {tg_op} table: {node.name}")
 
-        filters = {node.table: [], root.table: []}
-        extra = {}
+        filters: dict = {node.table: [], root.table: []}
+        extra: dict = {}
 
         if tg_op == INSERT:
 
@@ -717,11 +732,11 @@ class Sync(Base):
         ]
         """
         if filters.get(node.table):
-            _filters = []
-            keys = set([])
-            values = set([])
+            _filters: list = []
+            keys: Set = set([])
+            values: Set = set([])
             for _filter in filters.get(node.table):
-                where = []
+                where: list = []
                 for key, value in _filter.items():
                     where.append(getattr(node.model.c, key) == value)
                     keys.add(key)
@@ -731,22 +746,27 @@ class Sync(Base):
             if len(keys) == 1:
                 # If we have the same key then the node does not have a
                 # compound primary key
-                column = list(keys)[0]
+                column: list = list(keys)[0]
+                _values = []
+                for value in values:
+                    if (
+                        str(getattr(node.model.c, column).type).lower()
+                        == "uuid"
+                    ):
+                        _values.append((uuid.UUID(value),))
+                    else:
+                        _values.append(
+                            (
+                                getattr(node.model.c, column).type.python_type(
+                                    value
+                                ),
+                            )
+                        )
+
                 node._filters.append(
                     getattr(node.model.c, column).in_(
                         sa.select(
-                            Values(sa.column(column))
-                            .data(
-                                [
-                                    (
-                                        getattr(
-                                            node.model.c, column
-                                        ).type.python_type(value),
-                                    )
-                                    for value in values
-                                ]
-                            )
-                            .alias("x")
+                            Values(sa.column(column)).data(_values).alias("x")
                         )
                     )
                 )
@@ -761,11 +781,11 @@ class Sync(Base):
         extra: Optional[dict] = None,
     ) -> Generator:
         if filters is None:
-            filters = {}
+            filters: dict = {}
 
-        root = self.tree.build(self.nodes)
+        root: Node = self.tree.build(self.nodes)
 
-        self.query_builder.isouter = True
+        self.query_builder.isouter: bool = True
 
         for node in traverse_post_order(root):
 
@@ -804,8 +824,10 @@ class Sync(Base):
         if self.verbose:
             compiled_query(node._subquery, "Query")
 
+        count = self.fetchcount(node._subquery)
+
         with click.progressbar(
-            length=self.fetchcount(node._subquery),
+            length=count,
             show_pos=True,
             show_percent=True,
             show_eta=True,
@@ -819,7 +841,7 @@ class Sync(Base):
             ):
                 bar.update(1)
 
-                row = transform(row, self.nodes)
+                row: dict = transform(row, self.nodes)
                 row[META] = get_private_keys(keys)
                 if extra:
                     if extra["table"] not in row[META]:
@@ -869,7 +891,7 @@ class Sync(Base):
         """Save the current txid as the checkpoint."""
         if os.path.exists(self._checkpoint_file):
             with open(self._checkpoint_file, "r") as fp:
-                self._checkpoint = int(fp.read().split()[0])
+                self._checkpoint: int = int(fp.read().split()[0])
         return self._checkpoint
 
     @checkpoint.setter
@@ -884,9 +906,10 @@ class Sync(Base):
     def poll_redis(self) -> None:
         """Consumer which polls Redis continuously."""
         while True:
-            payloads = self.redis.bulk_pop()
+            payloads: dict = self.redis.bulk_pop()
             if payloads:
                 logger.debug(f"poll_redis: {payloads}")
+                self.count["redis"] += len(payloads)
                 self.on_publish(payloads)
             time.sleep(REDIS_POLL_INTERVAL)
 
@@ -902,18 +925,21 @@ class Sync(Base):
             psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT
         )
         cursor = conn.cursor()
-        channel = self.database
+        channel: str = self.database
         cursor.execute(f'LISTEN "{channel}"')
         logger.debug(f'Listening for notifications on channel "{channel}"')
 
         i: int = 0
-        j: int = 0
-
         while True:
             # NB: consider reducing POLL_TIMEOUT to increase throughout
             if select.select([conn], [], [], POLL_TIMEOUT) == ([], [], []):
                 if i % 10 == 0:
-                    sys.stdout.write(f"Polling db {channel}: {j:,} item(s)\n")
+                    sys.stdout.write(
+                        f"Syncing {channel} "
+                        f"db: [{self.count['db']:,}] => "
+                        f"redis: [{self.count['redis']:,}] => "
+                        f"elastic: [{self.count['elastic']:,}]...\n"
+                    )
                     sys.stdout.flush()
                 i += 1
                 continue
@@ -925,11 +951,11 @@ class Sync(Base):
                 os._exit(-1)
 
             while conn.notifies:
-                notification = conn.notifies.pop(0)
+                notification: AnyStr = conn.notifies.pop(0)
                 payload = json.loads(notification.payload)
                 self.redis.push(payload)
                 logger.debug(f"on_notify: {payload}")
-                j += 1
+                self.count["db"] += 1
             i = 0
 
     def on_publish(self, payloads: dict) -> None:
@@ -941,6 +967,7 @@ class Sync(Base):
         Deserialize the payload from Redis and sync to Elasticsearch.
         """
         logger.debug(f"on_publish len {len(payloads)}")
+        self.count["elastic"] += len(payloads)
 
         # Safe inserts are insert operations that can be performed in any order
         # Optimize the safe INSERTS
@@ -948,7 +975,7 @@ class Sync(Base):
         # if all payload operations are INSERTS
         if set(map(lambda x: x["tg_op"], payloads)) == set([INSERT]):
 
-            _payloads: dict = collections.defaultdict(list)
+            _payloads: dict = defaultdict(list)
 
             for payload in payloads:
                 _payloads[payload["table"]].append(payload)
@@ -958,10 +985,10 @@ class Sync(Base):
 
         else:
 
-            _payloads: List = []
+            _payloads: list = []
             for i, payload in enumerate(payloads):
                 _payloads.append(payload)
-                j = i + 1
+                j: int = i + 1
                 if j < len(payloads):
                     payload2 = payloads[j]
                     if (
@@ -972,13 +999,12 @@ class Sync(Base):
                         _payloads = []
                 elif j == len(payloads):
                     self.sync(self._payloads(_payloads))
-                    _payloads = []
+                    _payloads: list = []
 
-        txids = set(map(lambda x: x["xmin"], payloads))
+        txids: Set = set(map(lambda x: x["xmin"], payloads))
         # for truncate, tg_op txids is None so skip setting the checkpoint
         if txids != set([None]):
-            txmin = min(min(txids), self.txid_current) - 1
-            self.checkpoint = txmin
+            self.checkpoint: int = min(min(txids), self.txid_current) - 1
 
     def pull(self) -> None:
         """Pull data from db."""
@@ -987,10 +1013,10 @@ class Sync(Base):
         logger.debug(f"pull txmin: {txmin} txmax: {txmax}")
         # forward pass sync
         self.sync(self._sync(txmin=txmin, txmax=txmax))
-        self.checkpoint = txmax or self.txid_current
+        self.checkpoint: int = txmax or self.txid_current
         # now sync up to txmax to capture everything we may have missed
         self.logical_slot_changes(txmin=txmin, txmax=txmax)
-        self._truncate = True
+        self._truncate: bool = True
 
     @threaded
     def truncate_slots(self) -> None:
@@ -1095,7 +1121,7 @@ def main(
         sys.stdout.write(f"Version: {__version__}\n")
         return
 
-    params = {
+    kwargs: dict = {
         "user": user,
         "host": host,
         "port": port,
@@ -1103,24 +1129,22 @@ def main(
         "sslrootcert": sslrootcert,
     }
     if password:
-        params["password"] = click.prompt(
+        kwargs["password"] = click.prompt(
             "Password",
             type=str,
             hide_input=True,
         )
-    params = {key: value for key, value in params.items() if value is not None}
+    kwargs: dict = {
+        key: value for key, value in kwargs.items() if value is not None
+    }
 
-    config = get_config(config)
+    config: str = get_config(config)
 
-    show_settings(config, params)
+    show_settings(config, **kwargs)
 
     with Timer():
         for document in json.load(open(config)):
-            sync = Sync(
-                document,
-                verbose=verbose,
-                params=params,
-            )
+            sync = Sync(document, verbose=verbose, **kwargs)
             sync.pull()
             if daemon:
                 sync.receive()
