@@ -2,13 +2,14 @@
 import logging
 import os
 import sys
-import threading
-import time
 from datetime import timedelta
+from threading import Thread
+from time import time
 from typing import Optional
 from urllib.parse import quote_plus
 
 from .exc import SchemaError
+from .plugin import Plugins
 from .settings import (
     ELASTICSEARCH_HOST,
     ELASTICSEARCH_PASSWORD,
@@ -32,9 +33,9 @@ logger = logging.getLogger(__name__)
 
 def timeit(func):
     def timed(*args, **kwargs):
-        since = time.time()
+        since = time()
         retval = func(*args, **kwargs)
-        until = time.time()
+        until = time()
         sys.stdout.write(
             f"{func.__name__} ({args}, {kwargs}) {until-since} secs\n"
         )
@@ -48,11 +49,11 @@ class Timer:
         self._message = message or ""
 
     def __enter__(self):
-        self.start = time.time()
+        self.start = time()
         return self
 
     def __exit__(self, *args):
-        self.end = time.time()
+        self.end = time()
         self.elapsed = self.end - self.start
         sys.stdout.write(
             f"{self._message} {str(timedelta(seconds=self.elapsed))} "
@@ -60,17 +61,21 @@ class Timer:
         )
 
 
-def show_settings(schema: str = None, params: dict = {}) -> None:
+def show_settings(schema: str = None, **kwargs) -> None:
     """Show configuration."""
     logger.info("\033[4mSettings\033[0m:")
     logger.info(f'{"Schema":<10s}: {schema or SCHEMA}')
     logger.info("-" * 65)
     logger.info("\033[4mPostgres\033[0m:")
     logger.info(
-        f'URL: postgresql://{params.get("user", PG_USER)}:*****@'
-        f'{params.get("host", PG_HOST)}:'
-        f'{params.get("port", PG_PORT)}'
+        f'URL: postgresql://{kwargs.get("user", PG_USER)}:*****@'
+        f'{kwargs.get("host", PG_HOST)}:'
+        f'{kwargs.get("port", PG_PORT)}'
     )
+    for key in kwargs:
+        if key == "password":
+            continue
+        logger.info(f"{key}: {kwargs[key]}")
     logger.info("\033[4mElasticsearch\033[0m:")
     if ELASTICSEARCH_USER:
         logger.info(
@@ -91,13 +96,22 @@ def threaded(fn):
     """Decorator for threaded code execution."""
 
     def wrapper(*args, **kwargs):
-        thread = threading.Thread(target=fn, args=args, kwargs=kwargs)
+        thread: Thread = Thread(target=fn, args=args, kwargs=kwargs)
         thread.start()
         return thread
 
     return wrapper
 
 
+def get_auth(key):
+    try:
+        plugins: Plugins = Plugins("plugins", ["Auth"])
+        return plugins.auth(key)
+    except ModuleNotFoundError as e:
+        logger.warning(f"ModuleNotFoundError: {e}")
+        return None
+
+
 def get_elasticsearch_url(
     scheme: Optional[str] = None,
     user: Optional[str] = None,
@@ -108,11 +122,15 @@ def get_elasticsearch_url(
     """
     Return the URL to connect to Elasticsearch.
     """
-    scheme = scheme or ELASTICSEARCH_SCHEME
-    host = host or ELASTICSEARCH_HOST
-    port = port or ELASTICSEARCH_PORT
-    user = user or ELASTICSEARCH_USER
-    password = password or ELASTICSEARCH_PASSWORD
+    scheme: str = scheme or ELASTICSEARCH_SCHEME
+    host: str = host or ELASTICSEARCH_HOST
+    port: str = port or ELASTICSEARCH_PORT
+    user: str = user or ELASTICSEARCH_USER
+    password: str = (
+        get_auth("ELASTICSEARCH_PASSWORD")
+        or password
+        or ELASTICSEARCH_PASSWORD
+    )
     if user:
         return f"{scheme}://{user}:{quote_plus(password)}@{host}:{port}"
     logger.debug("Connecting to Elasticsearch without authentication.")
@@ -129,10 +147,10 @@ def get_postgres_url(
     """
     Return the URL to connect to Postgres.
     """
-    user = user or PG_USER
-    host = host or PG_HOST
-    password = password or PG_PASSWORD
-    port = port or PG_PORT
+    user: str = user or PG_USER
+    host: str = host or PG_HOST
+    password: str = get_auth("PG_PASSWORD") or password or PG_PASSWORD
+    port: str = port or PG_PORT
     if not password:
         logger.debug("Connecting to Postgres without password.")
         return f"postgresql://{user}@{host}:{port}/{database}"
@@ -151,11 +169,11 @@ def get_redis_url(
     """
     Return the URL to connect to Redis.
     """
-    host = host or REDIS_HOST
-    password = password or REDIS_AUTH
+    host: str = host or REDIS_HOST
+    password: str = get_auth("REDIS_AUTH") or password or REDIS_AUTH
     port = port or REDIS_PORT
-    db = db or REDIS_DB
-    scheme = scheme or REDIS_SCHEME
+    db: str = db or REDIS_DB
+    scheme: str = scheme or REDIS_SCHEME
     if password:
         return f"{scheme}://:{quote_plus(password)}@{host}:{port}/{db}"
     logger.debug("Connecting to Redis without password.")
@@ -166,7 +184,7 @@ def get_config(config: Optional[str] = None) -> str:
     """
     Return the schema config for PGSync.
     """
-    config = config or SCHEMA
+    config: str = config or SCHEMA
     if not config:
         raise SchemaError(
             "Schema config not set\n. "
